#!/usr/bin/env python
#-*- coding:utf-8 -*-

import numpy as np
import ctypes as ct


def load_library():
    names = ["libkinectcalibration.so", "libkinectcalibration.dll"]
    for name in names:
        try:
            return ct.CDLL(name)
        except OSError: pass
    
    raise OSError("libkinectcalibration lib not found")
 
libcalibration = load_library()
transform = libcalibration.transform
transform.restype = None
transform.argtypes = [ct.c_void_p]*4
    
load_yml = libcalibration.load_yml
load_yml.restype = None
load_yml.argtypes = [ct.c_char_p]
    
load_yml("kinectcalibrationparams.yml")


def getPointCloud(video, depth):
    ''' Holt einen Frame von der Kinect und bereitet die Daten als
    Punktwolke auf.
    '''
    global transform
    
    points = np.empty((640*480, 3), dtype=np.float32)
    colors = np.empty((640*480, 3), dtype=np.uint8)
    transform(  video.ctypes.data, depth.ctypes.data, \
                points.ctypes.data, colors.ctypes.data)
    
    return points, colors
    
def getRegisteredColorImage(video, depth, filter_invalid = True):
    '''Liefert das, auf die Tiefeninformationen registrierte Farbbild
    zurück. Dazu wird die Funktion getPointCloud aufgerufen und einige
    Umformunge gemacht. Nicht transformierbare Pixel erhalten wahlweise 
    den Ferbwert [-1,-1, -1], falls filter_invalid==true'''
    points, colors = getPointCloud(video, depth)
    colors_filtered = colors.copy()
    colors_filtered = colors_filtered.reshape((480,640,3))
    
    if (filter_invalid):
        invalid_pixel = np.where(depth == depth.max())
        colors_filtered[invalid_pixel] = 0

    return colors_filtered