#ifndef __KINECT_POINT_SOURCE_H
#define __KINECT_POINT_SOURCE_H

#include <string>
#include <vector>

#include <cstddef>
#include <cv.h>
#include <cv.hpp>

#include <Eigen/Core>

class KinectPointSource {
    private:
        std::vector<float> z_lookup;
        std::string yml;
        
        cv::Mat depth_map_x;
        cv::Mat depth_map_y;
        cv::Mat rgb_map_x;
        cv::Mat rgb_map_y;
        
        Eigen::Matrix3f R;
        Eigen::Vector3f T;
        
        float fx_d, fy_d, cx_d, cy_d;
        float fx_rgb, fy_rgb, cx_rgb, cy_rgb;
        
        void initialize_z_lookup();
        void initialize_matrices();
        
    public:
        KinectPointSource(const std::string& yml);
        void remap_depth(const cv::Mat& src, cv::Mat& dst) const;
        void remap_rgb(const cv::Mat& src, cv::Mat& dst) const;
        void reproject(const cv::Mat& rgb, const cv::Mat& depth,
            std::vector<cv::Point3f> &points,
            std::vector<cv::Vec3b> &colors) const;
};
#endif

